<?php

namespace Drupal\maintenance_notifier\Service;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Utility\Token;
use Psr\Log\LoggerInterface;

/**
 * Class MaintenanceNotifier.
 *
 * Provides functionality to check maintenance mode status
 * and send email notifications.
 *
 * @package Drupal\maintenance_notifier\Service
 */
class MaintenanceNotifier {
  use StringTranslationTrait;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  public function __construct(
    StateInterface $state,
    ConfigFactoryInterface $configFactory,
    MailManagerInterface $mailManager,
    LanguageManagerInterface $languageManager,
    LoggerInterface $logger,
    TimeInterface $time,
    Token $token,
    EntityTypeManagerInterface $entityTypeManager,
  ) {
    $this->state = $state;
    $this->configFactory = $configFactory;
    $this->mailManager = $mailManager;
    $this->languageManager = $languageManager;
    $this->logger = $logger;
    $this->time = $time;
    $this->token = $token;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Check maintenance mode and send email notification.
   */
  public function checkAndSendNotification() {
    $config = $this->configFactory->get('maintenance_notifier.settings');
    $maintenance_mode = $this->state->get('system.maintenance_mode', FALSE);
    $last_maintenance_time = $this->state->get('maintenance_notifier.last_maintenance_time');

    if ($maintenance_mode) {
      if (!$last_maintenance_time) {
        // Set the last maintenance time if it's
        // the first time maintenance is on.
        $this->state->set('maintenance_notifier.last_maintenance_time', $this->time->getCurrentTime());
      }
      else {
        $time_in_maintenance = $this->time->getCurrentTime() - $last_maintenance_time;
        $threshold = $config->get('time_threshold') * 60;

        if ($time_in_maintenance >= $threshold) {
          // Send notification.
          $this->sendNotification();
        }
      }
    }
    else {
      // Reset the last maintenance time if maintenance is off.
      $this->state->set('maintenance_notifier.last_maintenance_time', NULL);
    }
  }

  /**
   * Sends the notification email.
   */
  protected function sendNotification() {
    $config = $this->configFactory->get('maintenance_notifier.settings');

    // Get email subject and body with token replacements.
    $subject = $config->get('email_subject');
    $body = $config->get('email_body');

    $tokens = [];
    $options = ['clear' => TRUE];
    $subject = $this->token->replace($subject, $tokens, $options);
    $body = $this->token->replace($body, $tokens, $options);

    // Get email addresses and send notifications.
    $recipients = $config->get('notification_email');
    if ($recipients) {
      $recipients = $this->parseEmailAddresses($recipients);
    }
    else {
      $roles = $config->get('notification_roles');
      $recipients = $this->getEmailsByRoles($roles);
    }

    // Get the maintenance start time from the state.
    $start_time = $this->state->get('maintenance_notifier.last_maintenance_time');
    $current_time = $this->time->getCurrentTime();

    // Calculate how long the site has been in maintenance mode.
    // Replace [duration] token with the formatted duration.
    // Check if the body contains the [duration] token.
    if (strpos($body, '[duration]') !== FALSE) {
      $duration_in_seconds = $current_time - $start_time;
      $hours = floor($duration_in_seconds / 3600);
      $minutes = floor(($duration_in_seconds % 3600) / 60);
      $seconds = $duration_in_seconds % 60;

      $duration = '';
      if ($hours > 0) {
        $duration .= $this->t('@count hour', ['@count' => $hours]) . ($hours === 1 ? '' : 's') . ', ';
      }
      if ($minutes > 0) {
        $duration .= $this->t('@count minute', ['@count' => $minutes]) . ($minutes === 1 ? '' : 's') . ', ';
      }
      $duration .= $this->t('@count second', ['@count' => $seconds]) . ($seconds === 1 ? '' : 's');

      $body = str_replace('[duration]', $duration, $body);
    }

    $params = [
      'subject' => $subject,
      'body' => $body,
    ];

    // Loop through recipients and send email.
    foreach ($recipients as $email) {
      $result = $this->mailManager->mail('maintenance_notifier', 'maintenance_notification', $email, $this->languageManager->getDefaultLanguage()->getId(), $params);

      if ($result['result'] !== TRUE) {
        $this->logger->error('Unable to send maintenance notification to %email', ['%email' => $email]);
      }
      else {
        $this->logger->info('Sent maintenance notification to %email', ['%email' => $email]);
      }
    }
  }

  /**
   * Parses a comma-separated list of email addresses into an array.
   *
   * @param string $emails
   *   Comma-separated list of email addresses.
   *
   * @return array
   *   Array of trimmed email addresses.
   */
  protected function parseEmailAddresses($emails) {
    return array_map('trim', explode(',', $emails));
  }

  /**
   * Get email addresses of users by roles.
   *
   * @param array $roles
   *   Array of roles.
   *
   * @return array
   *   Array of email addresses.
   */
  protected function getEmailsByRoles(array $roles) {
    $emails = [];
    foreach ($roles as $role) {
      $users = $this->entityTypeManager->getStorage('user')->loadByProperties(['roles' => $role]);
      /** @var \Drupal\user\UserInterface $user */
      foreach ($users as $user) {
        if ($user->getEmail()) {
          $emails[] = $user->getEmail();
        }
      }
    }
    return $emails;
  }

}
